import argparse
import sys
import os.path
import logging
import shutil
import random

from kadai import log, config_handler
from kadai.settings import DATA_PATH, CONFIG_PATH, CACHE_PATH
from kadai.utils import FileUtils
from kadai.themer import Themer

logger = log.setup_logger(__name__, log.defaultLoggingHandler(), level=logging.WARNING)

def get_args():
    """ Get the args parsed from the command line and does arg handling stuff """

    arg = argparse.ArgumentParser(description="Generate and switch wallpaper themes")

    arg.add_argument('-v', action='store_true',
        help='Verbose Logging')

    arg.add_argument("-g", action="store_true",
        help="generate themes")
    
    arg.add_argument("-i", metavar="\"path/to/dir\"",
        help="the input file")
    
    arg.add_argument("-p", action="store_true",
        help="Use last set theme")

    arg.add_argument('--override', action="store_true",
        help="Override exisiting themes")
    
    arg.add_argument("--clear", action="store_true",
        help="Clear all data relating to KADAI")

    arg.add_argument("--backend", metavar="name",
        help="vibrance/hue/k_means")

    arg.add_argument("--progress", action="store_true",
        help="Show progress of theme generation")

    arg.add_argument("--warranty", action="store_true",
        help="Show the programs warranty")

    arg.add_argument("--light", action="store_true",
        help="Enable light theme")

    return arg

def parse_args(parser):
    """
    Parses the arguments onto different actions

    Arguments:
        parser (idk) -- the argument parser
    """

    args = parser.parse_args()

    if len(sys.argv) <= 1:
        parser.print_help()
        sys.exit(1)

    VERBOSE_MODE = True if args.v else False

    # Determine flags from config and cli
    engine_type = config_handler.compareFlagWithConfig(args.backend, config_handler.parse_config()['engine'])
    show_progress = config_handler.compareFlagWithConfig(args.progress, config_handler.parse_config()['progress'])
    enable_light_theme = config_handler.compareFlagWithConfig(args.light, config_handler.parse_config()['light'])

    if args.warranty:
        print("""Copyright (C) 2020 slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.""")
        exit(0)

    if args.i:
        themer = Themer(args.i, DATA_PATH)
        themer.setEngine(engine_type)
        themer.setOverride(args.override)
        themer.disableProgress(not show_progress)
        if enable_light_theme:
            themer.enableLightTheme()

        if args.g:
            themer.generate()
            sys.exit(0)
        else:
            # If the theme file does not exist generate it and then update to it
            try:
                themer.update()
            except FileUtils.noPreGenThemeError:
                themer.generate()
                themer.update()
            sys.exit(0)
    elif args.p:
        # Check if the cached image exists, if it does update to that
        last_image = os.path.join(DATA_PATH, "image")
        if(FileUtils.check_if_image(last_image)):
            themer = Themer(last_image, DATA_PATH)
            if enable_light_theme:
                themer.enableLightTheme()
            themer.update()
        else:
            logger.critical("Last image invalid or does not exist!")
            sys.exit(1)
    
    elif args.clear:
        clear = input("Are you sure you want to remove the cache relating to KADAI? [y/N] ").lower()
        if(clear == "y"):
            try:
                shutil.rmtree(DATA_PATH)
            except:
                raise
            logger.info("Cleared KADAI cache folders")
        else:
            logger.info("Canceled clearing cache folders...")
    else:
        logger.critical("No file specified...")
        sys.exit(1)

def main():
    # Create required directories
    try:
        os.makedirs(CACHE_PATH, exist_ok=True)
        os.makedirs(DATA_PATH, exist_ok=True)
        os.makedirs(CONFIG_PATH, exist_ok=True)
    except:
        raise

    parser = get_args()
    parse_args(parser)

if __name__ == "__main__":
    main()

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""


import os
import re
import tqdm
import logging
import json
import random
import errno
from colorthief import ColorThief
from PIL import Image

from kadai.utils import FileUtils,ColorUtils
from kadai.settings import CONFIG_PATH
from kadai import log

logger = log.setup_logger(__name__+'.default', log.defaultLoggingHandler(), level=logging.WARNING)
tqdm_logger = log.setup_logger(__name__+'.tqdm', log.TqdmLoggingHandler(), level=logging.WARNING)

class Themer():
    def __init__(self, image_path, out_path):
        self.image_path = image_path
        self.out_path = out_path
        self.override = False
        self.run_hooks = True
        self.engine_name = 'vibrance'
        self.engine = getEngine(self.engine_name)
        self.theme_out_path = os.path.join(out_path, 'themes/')
        self.template_path = os.path.join(os.path.abspath(os.path.dirname(__file__)),
            "data/template.json")
        self.user_templates_path = os.path.join(CONFIG_PATH, 'templates/')
        self.disable_progress = True
        self.light_theme = False

        FileUtils.ensure_dir_exists(self.theme_out_path)

    def setImagePath(self, path):
        self.image_path = path

    def setOutPath(self, path):
        self.out_path = path

    def setEngine(self, engine_name):
        self.engine_name = engine_name
        self.engine = getEngine(engine_name)

    def setOverride(self, state):
        self.override = state
    
    def setUserTemplatePath(self, path):
        self.user_templates_path = path
    
    def setRunHooks(self, condition):
        self.run_hooks = condition

    def disableProgress(self, condition):
        self.disable_progress = condition

    def enableLightTheme(self):
        self.light_theme = True
    
    def generate(self):
        tmp_file = "/tmp/kadai-tmp.png"

        image_path_md5 = [[i, FileUtils.md5_file(i)[:20]] for i in FileUtils.get_image_list(self.image_path)]
        unprocessed_images = image_path_md5 if self.override else get_non_generated(image_path_md5, self.theme_out_path)

        if len(unprocessed_images) > 0:
            for i in tqdm.tqdm(range(len(unprocessed_images)), bar_format=log.bar_format, disable=self.disable_progress):
                image = unprocessed_images[i][0]
                md5_hash = unprocessed_images[i][1]
                out_file = os.path.join(self.theme_out_path, md5_hash + '.json')
            
                create_tmp_image(image, tmp_file)

                colors = self.engine(tmp_file).generate()
                pallete = createValueColorPallete(colors)

                tqdm_logger.log(15, "[" + str(i+1) + "/" + str(len(unprocessed_images)) + "] Generating theme for " + image + "...")
            
                createTemplateFromPallete(pallete, str(image), out_file)
                #with open(self.template_path) as template_file:
        else:
            logger.info("No themes to generate.")
        
    def update(self):
        if os.path.isdir(self.image_path):
            images = FileUtils.get_image_list(self.image_path)
            random.shuffle(images)
            self.image_path = images[0]
        elif not os.path.isfile(self.image_path):
            logger.critical("Provided file is not recognised!")
            exit(1)
        
        md5_hash = FileUtils.md5_file(self.image_path)[:20]

        if not os.path.isfile(os.path.join(self.theme_out_path, md5_hash+".json")):
            raise FileUtils.noPreGenThemeError("Theme file for this image does not exist!")

        with open(os.path.join(self.theme_out_path, md5_hash + ".json")) as json_data:
            theme_data = json.load(json_data)

        colors = theme_data['colors']
        primary_color = theme_data['primary']
        wallpaper = theme_data['wallpaper']

        if self.light_theme:
            theme_colors = makeLightThemeFromColors(colors)
        else:
            theme_colors = makeDarkThemeFromColors(colors)

        templates = get_template_files(self.user_templates_path)

        for template in templates:
            template_path = os.path.join(self.user_templates_path, template)
            out_file = os.path.join(self.out_path, template[:-5])
            createFileFromTemplate(template_path, out_file, theme_colors, primary_color)
        
        # Link wallpaper to cache folder
        linkWallpaperPathInFolder(wallpaper, self.out_path)

        # Run external scripts
        if self.run_hooks:
            FileUtils.run_hooks(light_theme=self.light_theme)

def getEngine(engine_name):
    if engine_name == "hue":
        from kadai.engine import HueEngine
        return HueEngine
    elif engine_name == "k_means":
        from kadai.engine import kMeansEngine
        return kMeansEngine
    else:
        from kadai.engine import VibranceEngine
        return VibranceEngine

def createValueColorPallete(colors):
    values = [0.1, 0.3, 0.5, 0.7, 0.9]
    pallete = {}
    for i in range(len(colors)):
        color_value_dictionary = createValueSpreadDictionary(values, colors[i])
        pallete['color{}'.format(i)] = color_value_dictionary
    return pallete

def createValueSpreadDictionary(values, color):
    pallete = {}
    for value in values:
        pallete[str(value)] =  ColorUtils.rgb_to_hex(ColorUtils.changeValueFromRGB(color, value))
    return pallete

def get_template_files(template_dir):
    # Get all templates in the templates folder
    templates = [f for f in os.listdir(template_dir)
        if re.match(r'.*\.base$', f)]

    return templates

def get_non_generated(images, theme_dir):
    ungenerated_images = []
    theme_dir = os.path.expanduser(theme_dir)
    for i in range(len(images)):
        md5_hash = images[i][1]

        if len([os.path.join(theme_dir, x.name) for x in os.scandir(theme_dir)\
            if md5_hash in x.name]) == 0:
            ungenerated_images.append(images[i])

    return ungenerated_images

def clearAndWriteDataToFile(file_path, data):
    if os.path.isfile(file_path):
        open(os.path.expanduser(file_path), 'w').close()
    with open(os.path.expanduser(file_path), 'a') as file:
        file.write(data)

def clearAndWriteJsonToFile(file_path, json_data):
    if os.path.isfile(file_path):
        open(os.path.expanduser(file_path), 'w').close()
    with open(os.path.expanduser(file_path), 'wb') as file:
        file.write(json.dumps(json_data,
            indent=4, separators=(',',': ')).encode('utf-8'))

def createTemplateFromPallete(pallete, image_path, out_path):
    file_contents = {}
    domiant_color = getDominantColorFromImage(image_path)
    file_contents['colors'] = pallete
    file_contents['wallpaper'] = image_path
    file_contents['primary'] = domiant_color

    clearAndWriteJsonToFile(out_path, file_contents)

def create_tmp_image(image, path):
    img = Image.open(image)
    image_out = img.resize((100,50), Image.NEAREST).convert('RGB')
    image_out.save(path)

def modifyFiledataWithTemplate(filedata, colors, primary_color):
    # Change placeholder values
    for i in range(len(colors)):
        filedata = filedata.replace("[color" + str(i) + "]", str(colors['color'+str(i)]))

    filedata = filedata.replace("[background]", str(colors['color0']))
    filedata = filedata.replace("[background_light]", str(colors['color8']))
    filedata = filedata.replace("[foreground]", str(colors['color15']))
    filedata = filedata.replace("[foreground_dark]", str(colors['color7']))
    filedata = filedata.replace("[primary]", str(primary_color))

    return filedata

def linkWallpaperPathInFolder(wallpaper, folder_path):
    image_symlink = os.path.join(folder_path, 'image')

    try:
        os.symlink(wallpaper, image_symlink)
    except OSError as e:
        if e.errno == errno.EEXIST:
            os.remove(image_symlink)
            os.symlink(wallpaper, image_symlink)
        else:
            raise e

def getDominantColorFromImage(image_path):
    tmp_image_path = '/tmp/kadai-tmp.png'
    create_tmp_image(image_path, tmp_image_path)
    color_cmd = ColorThief(tmp_image_path).get_palette
    raw_colors = color_cmd(color_count=2, quality=3)
    return ColorUtils.rgb_to_hex(ColorUtils.hsv_to_rgb(ColorUtils.changeHsvValue(ColorUtils.rgb_to_hsv(raw_colors[0]), 0.6)))

def createFileFromTemplate(template_path, out_file, colors, primary_color):
    with open(template_path) as template_file:
        filedata = template_file.read()
        filedata = modifyFiledataWithTemplate(filedata, colors, primary_color)

        clearAndWriteDataToFile(out_file, filedata)

def makeLightThemeFromColors(colors):
    new_colors = {}
    new_colors['color0'] = ColorUtils.rgb_to_hex(ColorUtils.changeSaturationFromRGB(ColorUtils.hex_to_rgb(colors['color0']['0.9']), 0.05))
    new_colors['color7'] = ColorUtils.rgb_to_hex(ColorUtils.changeSaturationFromRGB(ColorUtils.hex_to_rgb(colors['color0']['0.3']), 0.2))
    new_colors['color8'] = ColorUtils.rgb_to_hex(ColorUtils.changeSaturationFromRGB(ColorUtils.hex_to_rgb(colors['color0']['0.7']), 0.05))
    new_colors['color15'] = ColorUtils.rgb_to_hex(ColorUtils.changeSaturationFromRGB(ColorUtils.hex_to_rgb(colors['color0']['0.1']), 0.2))
    for i in range(6):
        new_colors['color{}'.format(str(i+1))] = colors['color{}'.format(str(1+i))]['0.5']
        new_colors['color{}'.format(str(i+9))] = colors['color{}'.format(str(1+i))]['0.3']
    return new_colors

def makeDarkThemeFromColors(colors):
    new_colors = {}
    new_colors['color0'] = ColorUtils.rgb_to_hex(ColorUtils.changeSaturationFromRGB(ColorUtils.hex_to_rgb(colors['color0']['0.1']), 0.2))
    new_colors['color7'] = ColorUtils.rgb_to_hex(ColorUtils.changeSaturationFromRGB(ColorUtils.hex_to_rgb(colors['color0']['0.7']), 0.05))
    new_colors['color8'] = ColorUtils.rgb_to_hex(ColorUtils.changeSaturationFromRGB(ColorUtils.hex_to_rgb(colors['color0']['0.3']), 0.2))
    new_colors['color15'] = ColorUtils.rgb_to_hex(ColorUtils.changeSaturationFromRGB(ColorUtils.hex_to_rgb(colors['color0']['0.9']), 0.05))
    for i in range(6):
        new_colors['color{}'.format(str(i+1))] = colors['color{}'.format(str(1+i))]['0.7']
        new_colors['color{}'.format(str(i+9))] = colors['color{}'.format(str(1+i))]['0.9']
    return new_colors

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""


import os.path

if "XDG_DATA_HOME" in os.environ:
	DATA_PATH = os.path.join(os.getenv("XDG_DATA_HOME"), 'kadai')
else:
	DATA_PATH = os.path.expanduser('~/.local/share/kadai')

if "XDG_CONFIG_HOME" in os.environ:
	CONFIG_PATH = os.path.join(os.getenv("XDG_CONFIG_HOME"), 'kadai')
else:
	CONFIG_PATH = os.path.expanduser('~/.config/kadai')

if "XDG_CACHE_HOME" in os.environ:
	CACHE_PATH = os.path.join(os.getenv("XDG_CACHE_HOME"), 'kadai')
else:
	CACHE_PATH = os.path.expanduser('~/.cache/kadai')

DEBUG_MODE = False

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""


from PIL import Image
import math
import random

class kMeansEngine():
    def __init__(self, image):
        self.image = image
        self.colors = get_colors(image)

    def generate(self):
        return self.colors

class Point:
    def __init__(self, coordinates):
        self.coordinates = coordinates    

class Cluster:
    def __init__(self, center, points):
        self.center = center
        self.points = points

class KMeans:
    def __init__(self, n_clusters, min_diff = 10):
        self.n_clusters = n_clusters
        self.min_diff = min_diff


    def calculate_center(self, points):
        n_dim = len(points[0].coordinates)
        vals = [0.0 for i in range(n_dim)]

        for p in points:
            for i in range(n_dim):
                vals[i] += p.coordinates[i]
        
        coords = [(v/len(points)) for v in vals]
        return Point(coords)

    def assign_points(self, clusters, points):
        plists = [[] for i in range(self.n_clusters)]

        for p in points:
            smallest_distance = float('inf')

            for i in range(self.n_clusters):
                distance = euclidan(p, clusters[i].center)
                if distance < smallest_distance:
                    smallest_distance = distance
                    idx = i
            
            plists[idx].append(p)

        return plists

    
    def fit(self, points):
        clusters = [Cluster(center=p, points=[p]) for p in random.sample(points, self.n_clusters)]

        while True:
            plists = self.assign_points(clusters, points)
            diff=0

            for i in range(self.n_clusters):
                if not plists[i]:
                    continue

                old = clusters[i]
                center = self.calculate_center(plists[i])
                new = Cluster(center, plists[i])
                clusters[i] = new
                diff = max(diff, euclidan(old.center, new.center))

            if diff < self.min_diff:
                break

        return clusters

def get_points(image_path):
    img = Image.open(image_path)
    w,h = img.size

    points = []
    for count, color in img.getcolors(w*h):
        for _ in range(count):
            points.append(Point(color))

    return points

def euclidan(p, q):
    n_dim = len(p.coordinates)
    return math.sqrt(sum([
        (p.coordinates[i] - q.coordinates[i]) ** 2 for i in range(n_dim)
    ]))

def get_colors(filename, n_colors=7):
    points = get_points(filename)
    clusters = KMeans(n_clusters=n_colors).fit(points)
    clusters.sort(key=lambda c: len(c.points), reverse = True)
    rgbs = list(map(tuple, [map(int, c.center.coordinates) for c in clusters]))
    return rgbs

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""


from kadai.engine.vibrance import VibranceEngine
from kadai.engine.genhue import HueEngine
from kadai.engine.k_means import kMeansEngine

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""


import colorsys
from PIL import Image, ImageDraw

def rgb_to_hex(color):
	"""
	Convert an rgb color to hex.

	Arguments:
		color (list) -- list of red, green, and blue for a color [r, g, b]
	"""

	return "#%02x%02x%02x" % (*color,)

def hex_to_rgb(color):
	"""
	Convert a hex color to rgb.
	Arguments:
		color (string) -- hexadecimal value with the leading '#'
	"""

	return tuple(bytes.fromhex(color.strip("#")))

def rgb_to_hsv(color):
	"""
	Converts from rgb to hsv

	Arguments:
		color (list) -- list of red, green, and blue for a color [r, g, b]
	"""
	new_cols = list(colorsys.rgb_to_hsv(*[float(x/255) for x in color]))
	return tuple(new_cols)

def hsv_to_rgb(color):
	"""
	Converts from hsv to rgb

	Arguments:
		color (list) -- list of hue, saturation, and value for a color [h, s, v]
	"""

	color_rgb = [col for col in colorsys.hsv_to_rgb(*color)]
	return tuple([int(col*255) for col in color_rgb])

def changeHsvValue(color, value):
    return (color[0], color[1], value)

def changeHsvHue(color, hue):
    return (hue, color[1], color[2])

def changeHsvSaturation(color, saturation):
    return (color[0], saturation, color[2])

def changeHueFromRGB(color, hue):
	hsv_color = rgb_to_hsv(color)
	hsv_color = (hue, hsv_color[1], hsv_color[2])
	return hsv_to_rgb(hsv_color)

def changeValueFromRGB(color, value):
	hsv_color = rgb_to_hsv(color)
	hsv_color = (hsv_color[0], hsv_color[1], value)
	return hsv_to_rgb(hsv_color)

def changeSaturationFromRGB(color, saturation):
	hsv_color = rgb_to_hsv(color)
	hsv_color = (hsv_color[0], saturation, hsv_color[2])
	return hsv_to_rgb(hsv_color)

def getHueFromRGB(color):
	hsv_color = rgb_to_hsv(color)
	return hsv_color[0]

def generate_pallete_image(colors, n_colors=7):
    x_pos = 0
    pallete_length=60
    palletes_length = pallete_length*n_colors

    img = Image.new('RGB', (palletes_length,30), color=(0,0,0))
    draw_image = ImageDraw.Draw(img)

    for color in colors:
        draw_image.rectangle(((x_pos, 0), (x_pos+pallete_length, 30)), fill=color)
        x_pos += pallete_length
    img.save('out.png')



import hashlib
import os
import re
import subprocess
from PIL import Image

from kadai.settings import CONFIG_PATH

class noPreGenThemeError(Exception):
    pass

def md5(string):
	"""
	Generates a md5 hash based on the parsed string

	Arguments:
		string (str) -- a string to be encoded
	"""

	hash_md5 = hashlib.md5(str(string).encode())
	return hash_md5.hexdigest()

def md5_file(fname):
	"""
	Generates a md5 hash based on the file parsed

	Arguments:
		fname (str) -- location of the file ('/home/bob/pic.png')
	"""

	hash_md5 = hashlib.md5()
	with open(fname, "rb") as f:
		for chunk in iter(lambda: f.read(4096), b""):
			hash_md5.update(chunk)
	return hash_md5.hexdigest()

def ensure_dir_exists(directory):
	try:
		os.makedirs(directory, exist_ok=True)
	except: raise

def get_image(image):
	"""
	Get the absolute path of a passed file (image)

	Arguments:
		image (str) -- location of the file
	"""
	if os.path.isfile(image): 
		return os.path.abspath(image)

def check_if_image(file):
	try:
		Image.open(file).verify()
		return True
	except:
		return False

def get_dir_imgs(img_dir):
	"""
	Get a list of all images in a directory

	Arguments:
		img_dir (str) -- the directory where the images are stored
	"""
	file_types = ("png", "jpg", "jpeg")
	return [get_image(os.path.join(img_dir, img.name)) for img in os.scandir(img_dir)
			if img.name.lower().endswith(file_types) and check_if_image(os.path.join(img_dir, img.name))]

def get_image_list(image_path):
	image_path = os.path.expanduser(image_path)
	if os.path.isfile(image_path):
		if(check_if_image(image_path)):
			return [get_image(image_path)]
		else:
			raise "Specified file is not an image!"
	elif os.path.isdir(image_path):
		images = get_dir_imgs(image_path)
		if len(images) == 0:
			raise "Specified directory does not contain any images!"

		return images
	else:
		raise ValueError("Unknown file type!")


def get_hooks(hooks_dir=os.path.join(CONFIG_PATH, 'hooks')):
	try:
		os.makedirs(hooks_dir, exist_ok=True)
	except:
		raise

	scripts = [f for f in os.listdir(hooks_dir) 
		if re.match(r'^([0-9]{2}-\w+)', f) 
			and os.access(os.path.join(hooks_dir, f), os.X_OK)]
	scripts.sort()
	return scripts

def run_hooks(light_theme=False, hooks_dir=os.path.join(CONFIG_PATH, 'hooks')):		
	scripts = get_hooks(hooks_dir)

	for script in scripts:
		script = os.path.join(hooks_dir, script)

		with open(os.devnull, 'w') as devnull:
			subprocess.run([script, str(light_theme).lower()], stdout=devnull, stderr=subprocess.STDOUT)


import math
import sys
import logging
from PIL import Image,ImageStat
from colorthief import ColorThief

from kadai import log
from kadai.utils import ColorUtils

logger = log.setup_logger(__name__, log.defaultLoggingHandler(), level=logging.WARNING)

class VibranceEngine():
    def __init__(self, image_path):
        self.raw_colors = gen_colors(image_path)
        self.image_path = image_path

    def generate(self):
        """
        Generate the palette. Returns a list of hexidecimal colors
        
        Arguments:
            img (str) -- location of the image
        """
        return sort_colors(self.raw_colors)

def get_image_brightness(im_file):
	im = Image.open(im_file)
	stat = ImageStat.Stat(im)
	r,g,b = stat.mean
	return math.sqrt(0.299*(r**2) + 0.587*(g**2) + 0.114*(b**2))

def sort_by_vibrance(colors):
	"""
	Sorts the colors by their vibrance (saturation * brightness(value))

	Arguments:
		colors (list) -- list of rgb colors
	"""

	hsv_distance=calculateVibranceFromList(colors)
	adj_colors = sorted(hsv_distance, key = lambda x:abs(x[1]-1))
	return [i[0] for i in adj_colors]

def calculateVibrance(color):
	hsv_color = [*ColorUtils.rgb_to_hsv(color)]
	ideal_brightness=1

	# Basically the closer the brightness is to the ideal brightness and
	# the higher the saturation is the larger the output value
	return hsv_color[1]*(2+(1-((hsv_color[2]/ideal_brightness)+(ideal_brightness/hsv_color[2]))))	

def calculateVibranceFromList(colors):
	hsv_distance = []
	for i in range(len(colors)):
		vibrance = calculateVibrance(colors[i])
		hsv_distance.append([colors[i], vibrance])
	return hsv_distance

def sort_to_list(colors, color_list):
	return [i for i in color_list if i in colors]

def sort_colors(colors):
	"""
	Sorts the colors based on a sorting algorithim, and returns a list of colors (length of 8)

	Arguments:
		colors (list) -- list of rgb formatted colors
	"""

	# Sort by vibrance and get the least vibrant and the 7 most vibrant
	sorted_colors = sort_by_vibrance(colors)
	top_vibrant = sorted_colors[:7]
	return sort_to_list(top_vibrant, colors)

def gen_colors(image_path):
	"""
	Create a list of colors, max of 16 and min of 8
	
	Arguments:
		img (str) -- location of the image
	"""

	color_cmd = ColorThief(image_path).get_palette
	raw_colors = color_cmd(color_count=16, quality=3)

	if len(raw_colors) <= 8:		
		logger.warn("ColorThief couldn't generate a suitable pallete")
		return None

	return raw_colors

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""


import colorsys

from PIL import Image,ImageStat
from colorthief import ColorThief

from kadai.utils import ColorUtils

# Match XColors
color_hues = (240, 0, 120, 60, 240, 300, 180)

class HueEngine():
    def __init__(self, image):
        self.image = image
        self.color = getDominantColorFromImage(image)

    def generate(self):
        distance = getMinDistanceFromHues(self.color)
        base_colors = shiftHuesByDistance(generateBaseColors(self.color), distance)
        return base_colors

def generateBaseColors(color):
    new_colors = []
    for i in range(len(color_hues)):
        hsv_color = ColorUtils.rgb_to_hsv(color)
        hsv_color = ColorUtils.changeHsvHue(hsv_color, float(color_hues[i]/360))
        if hsv_color[1] < 0.4:
            hsv_color = ColorUtils.changeHsvSaturation(hsv_color, 0.4)
        new_colors.append(ColorUtils.hsv_to_rgb(hsv_color))
    return new_colors


def getDominantColorFromImage(image_path):
    color_cmd = ColorThief(image_path).get_palette
    raw_colors = color_cmd(color_count=2, quality=3)
    return ColorUtils.hsv_to_rgb(ColorUtils.changeHsvValue(ColorUtils.rgb_to_hsv(raw_colors[0]), 0.7))

def shiftHuesByDistance(colors, distance):
    new_colors = []
    for color in colors:
        hsv_color = ColorUtils.rgb_to_hsv(color)
        new_hue = hsv_color[0] + distance
        if new_hue >= 1:
            new_hue = new_hue-1
        elif new_hue < 0 :
            new_hue = new_hue+1
        new_colors.append(ColorUtils.changeHueFromRGB(color, new_hue))
    return new_colors

def getMinDistanceFromHues(color):
    distances = []
    distances_positive = []
    for color_hue in color_hues:
        hsv_color = ColorUtils.rgb_to_hsv(color)

        distance = color_hue - (hsv_color[0]*360)
        distances.append(distance)
        distances_positive.append(abs(distance))
    
    min_distance_pos = distances_positive.index(min(distances_positive))
    return (distances[min_distance_pos]/360)

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""


from setuptools import setup

LONG_DESC = open('README.md').read()

setup(name='KADAI',
	version='1.0.0rc1',
	description='Simple wallpaper manager for tiling window managers.',
	long_description_content_type="text/markdown",
	long_description=LONG_DESC,
	classifiers=[
		'Development Status :: 4 - Beta',
		'Environment :: X11 Applications',
		'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
		'Natural Language :: English',
		'Operating System :: POSIX :: Linux',
		'Programming Language :: Python :: 3.8',
	],
	url='http://github.com/slapelachie/kadai',
	author='slapelachie',
	author_email='slapelachie@gmail.com',
	license='GPLv2',
	packages=['kadai', 'kadai.engine', 'kadai.utils'],
	entry_points={"console_scripts": ["kadai=kadai.__main__:main"]},
	include_package_data=True,
	install_requires = [
		'Pillow',
		'tqdm',
		'colorthief'
	],
	zip_safe=False)



import logging
import tqdm
import sys

default_format = ("[%(levelname)s\033[0m] "
	"\033[1;31m%(module)s\033[0m: "
	"%(message)s")

bar_format='{percentage:3.0f}% {n}/{total}'

class TqdmLoggingHandler(logging.Handler):
	def __init__(self, level=logging.NOTSET):
		super().__init__(level)

	def emit(self, record):
		try:
			self.setFormatter(logging.Formatter(default_format))
			msg = self.format(record)
			tqdm.tqdm.write(msg)
			self.flush()
		except (KeyboardInterrupt, SystemExit):
			raise
		except:
			self.handleError(record)

class defaultLoggingHandler(logging.StreamHandler):
	def emit(self, record):
		try:
			self.setFormatter(logging.Formatter(default_format+'\n'))
			msg = self.format(record)
			stream = self.stream
			stream.write(msg)
			self.flush()
		except (KeyboardInterrupt, SystemExit):
			raise
		except:
			self.handleError(record)	

def setup_logger(name, handler, level=logging.WARNING):
	""" Sets up the logger """	
	logger = logging.getLogger(name)
	logger.setLevel(level)
	logger.addHandler(handler)
	return logger

logging.addLevelName(logging.ERROR, '\033[1;31mE')
logging.addLevelName(logging.INFO, '\033[1;32mI')
logging.addLevelName(logging.WARNING, '\033[1;33mW')
logging.addLevelName(logging.CRITICAL, '\033[1;33mC')
logging.addLevelName(logging.DEBUG, '\033[1;33mD')

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""


import copy
import os
import json
import pickle
from kadai import settings

config_file_path = os.path.join(settings.CONFIG_PATH, 'config.json')

config_default = {
    'engine': 'vibrance',
    'out_directory': '$HOME/.local/share/',
    'light_theme': False,
    'progress': False,
    'debug': False
}

def parse_config():
    config = copy.copy(config_default)
    
    try:
        with open(config_file_path) as config_file:
            loaded_config = json.load(config_file)
            config.update(loaded_config)
    except IOError:
        save_config(config)
    
    return config

def save_config(config_dict):
    config_path = os.path.dirname(config_file_path)
    if not os.path.isdir(config_path):
        os.mkdir(config_path)

    with open(config_file_path, 'wb') as config_file:
        config_file.write(json.dumps(config_dict,
            indent=4, separators=(',',': ')).encode('utf-8'))

def load_config(config_file_path):
    with open(config_file_path, 'rb') as config_file:
        return pickle.load(config_file, encoding='utf-8')

def compareFlagWithConfig(flag, config_option):
    if(flag):
        return flag
    else:
        return config_option

"""
kadai - Simple wallpaper manager for tiling window managers.
Copyright (C) 2020  slapelachie

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Find the full license in the root of this project
"""